@@ -19,7 +19,8 @@
 #' If NULL, all rows are read.
 #' @param check.names logical. If TRUE then the names of the variables in the data frame 
 #' are checked to ensure that they are syntactically valid variable names
-#' @param namedRegion A named region in the Workbook. If not NULL startRow, rows and cols paramters are ignored.
+#' @param sep.names One character which substitutes blanks in column names. By default, "."
+#' @param namedRegion A named region in the Workbook. If not NULL startRow, rows and cols parameters are ignored.
 #' @param na.strings A character vector of strings which are to be interpreted as NA. Blank cells will be returned as NA.
 #' @param fillMergedCells If TRUE, the value in a merged cell is given to all cells within the merge.
 #' @param skipEmptyCols If \code{TRUE}, empty columns are skipped.
@@ -31,7 +32,8 @@
 #' @return data.frame
 #' @export
 #' @examples
-#' xlsxFile <- system.file("readTest.xlsx", package = "openxlsx")
+#' 
+#' xlsxFile <- system.file("extdata","readTest.xlsx", package = "openxlsx")
 #' df1 <- read.xlsx(xlsxFile = xlsxFile, sheet = 1, skipEmptyRows = FALSE)
 #' sapply(df1, class)
 #' 
@@ -45,19 +47,21 @@
 #' sapply(df2, class)
 #' head(df2)
 #' 
-#' wb <- loadWorkbook(system.file("readTest.xlsx", package = "openxlsx"))
+#' wb <- loadWorkbook(system.file("extdata","readTest.xlsx", package = "openxlsx"))
 #' df3 <- read.xlsx(wb, sheet = 2, skipEmptyRows = FALSE, colNames = TRUE)
 #' df4 <- read.xlsx(xlsxFile, sheet = 2, skipEmptyRows = FALSE, colNames = TRUE)
 #' all.equal(df3, df4)
 #' 
-#' wb <- loadWorkbook(system.file("readTest.xlsx", package = "openxlsx"))
+#' wb <- loadWorkbook(system.file("extdata",readTest.xlsx", package = "openxlsx"))
 #' df3 <- read.xlsx(wb, sheet = 2, skipEmptyRows = FALSE,
 #'  cols = c(1, 4), rows = c(1, 3, 4))
 #' 
 #' ## URL
 #' ## 
-#' #xlsxFile <- "https://github.com/awalker89/openxlsx/raw/master/inst/readTest.xlsx"
-#' #head(read.xlsx(xlsxFile))
+#' \dontrun{
+#'  xlsxFile <- "https://github.com/awalker89/openxlsx/raw/master/inst/readTest.xlsx"
+#'  head(read.xlsx(xlsxFile))
+#'  }
 #' 
 #' 
 #' @export
@@ -72,6 +76,7 @@ read.xlsx <- function(xlsxFile,
                       rows = NULL,
                       cols = NULL,
                       check.names = FALSE,
+                      sep.names = ".",
                       namedRegion = NULL,
                       na.strings = "NA",
                       fillMergedCells = FALSE){
@@ -92,6 +97,7 @@ read.xlsx.default <- function(xlsxFile,
                               rows = NULL,
                               cols = NULL,
                               check.names = FALSE,
+                              sep.names = ".",
                               namedRegion = NULL,
                               na.strings = "NA",
                               fillMergedCells = FALSE){
@@ -121,6 +127,9 @@ read.xlsx.default <- function(xlsxFile,
   if(!is.logical(check.names))
     stop("check.names must be TRUE/FALSE.")
   
+  if(!is.character(sep.names) | nchar(sep.names)!=1)
+    stop("sep.names must be a character and only one.")
+  
   if(length(sheet) > 1)
     stop("sheet must be of length 1.")
   
@@ -138,7 +147,7 @@ read.xlsx.default <- function(xlsxFile,
   }
   
   ## create temp dir and unzip
-  xmlDir <- file.path(tempdir(), "_excelXMLRead")
+  xmlDir <- file.path(tempdir(),paste0(sample(LETTERS,10),collapse=""),"_excelXMLRead")
   xmlFiles <- unzip(xlsxFile, exdir = xmlDir)
   
   on.exit(unlink(xmlDir, recursive = TRUE), add = TRUE)
@@ -149,15 +158,25 @@ read.xlsx.default <- function(xlsxFile,
   
   ## get workbook names
   workbookRelsXML <- paste(readLines(workbookRelsXML, warn = FALSE, encoding = "UTF-8"), collapse = "")
-  workbookRelsXML <- .Call("openxlsx_getChildlessNode", workbookRelsXML, "<Relationship ", PACKAGE="openxlsx")
+  workbookRelsXML <- getChildlessNode(xml = workbookRelsXML, tag = "<Relationship ")
   
   workbook <- unlist(readLines(workbook, warn = FALSE, encoding = "UTF-8"))
   workbook <- removeHeadTag(workbook)
-  sheets <- unlist(regmatches(workbook, gregexpr("<sheet .*/sheets>", workbook, perl = TRUE)))
+  
+  sheets <- unlist(regmatches(workbook, gregexpr("(?<=<sheets>).*(?=</sheets>)", workbook, perl = TRUE)))
+  sheets <- unlist(regmatches(sheets, gregexpr("<sheet[^>]*>", sheets, perl=TRUE)))
+  ## Some veryHidden sheets do not have a sheet content and their rId is empty.
+  ## Such sheets need to be filtered out because otherwise their sheet names
+  ## occur in the list of all sheet names, leading to a wrong association
+  ## of sheet names with sheet indeces.
+  sheets <- grep('r:id="[[:blank:]]*"', sheets, invert = TRUE, value = TRUE)
+  
   
   ## make sure sheetId is 1 based
   sheetrId <- unlist(getRId(sheets))
   sheetNames <- unlist(regmatches(sheets, gregexpr('(?<=name=")[^"]+', sheets, perl = TRUE)))
+  sheetNames <- replaceXMLEntities(sheetNames)
+  
   
   nSheets <- length(sheetrId)
   if(nSheets == 0)
@@ -165,9 +184,10 @@ read.xlsx.default <- function(xlsxFile,
   
   
   ## Named region logic
+  reading_named_region <- FALSE
   if(!is.null(namedRegion)){
     
-    dn <- .Call("openxlsx_getNodes", workbook, "<definedNames>", PACKAGE = "openxlsx")
+    dn <- getNodes(xml = workbook, tagIn = "<definedNames>")
     dn <- unlist(regmatches(dn, gregexpr("<definedName [^<]*", dn, perl = TRUE)))
     
     if(length(dn) == 0){
@@ -185,8 +205,9 @@ read.xlsx.default <- function(xlsxFile,
     dn <- dn[ind] 
     region <- regmatches(dn, regexpr('(?<=>)[^\\<]+', dn, perl = TRUE))
     sheet <- sheetNames[sapply(sheetNames, function(x) grepl(x, dn))]
+    if(length(sheet) > 1)
+      sheet <- sheet[which.max(nchar(sheet))]
     
-    region <- gsub(sheet, "", region, fixed = TRUE)
     region <- gsub("[^A-Z0-9:]", "", gsub(sheet, "", region, fixed = TRUE))
     
     if(grepl(":", region, fixed = TRUE)){
@@ -202,7 +223,7 @@ read.xlsx.default <- function(xlsxFile,
     }
     
     startRow <- 1
-    
+    reading_named_region <- TRUE
   }
   
   
@@ -212,12 +233,13 @@ read.xlsx.default <- function(xlsxFile,
   ## get the file_name for each sheetrId
   file_name <- sapply(sheetrId, function(rId){
     txt <- workbookRelsXML[grepl(sprintf('Id="%s"', rId), workbookRelsXML, fixed = TRUE)]
-    regmatches(txt, regexpr('(?<=Target=").+xml', txt, perl = TRUE))
+    regmatches(txt, regexpr('(?<=Target=").+xml(?=")', txt, perl = TRUE))
   })
   
   
   ## get the correct sheets
   if("character" %in% class(sheet)){
+    sheetNames <- replaceXMLEntities(sheetNames)
     sheetInd <- which(sheetNames == sheet)
     if(length(sheetInd) == 0)
       stop(sprintf('Cannot find sheet named "%s"', sheet))
@@ -228,14 +250,20 @@ read.xlsx.default <- function(xlsxFile,
     sheet <- file_name[sheet]
   }
   
+  if(length(sheet) == 0)
+    stop("Length of sheet is 0 - something has gone terribly wrong! Please report this bug on github (https://github.com/awalker89/openxlsx/issues) with an example xlsx file.")
+  
   ## get file
-  worksheet <- xmlFiles[grepl(sheet, xmlFiles, ignore.case = TRUE)]
+  worksheet <- xmlFiles[grepl(pattern = tolower(sheet), x = tolower(xmlFiles), fixed = TRUE)]
+  if(length(worksheet) == 0)
+    stop("Length of worksheet is 0 - something has gone terribly wrong! Please report this bug on github (https://github.com/awalker89/openxlsx/issues) with an example xlsx file.")
+  
   
   ## read in sharedStrings
   if(length(sharedStringsFile) > 0){
     sharedStrings <- getSharedStringsFromFile(sharedStringsFile = sharedStringsFile, isFile = TRUE)
     if(!is.null(na.strings)){
-      sharedStrings[sharedStrings %in% na.strings] <- "openxlsx_na_vlu"
+      sharedStrings[is.na(sharedStrings) | sharedStrings %in% na.strings] <- "openxlsx_na_vlu"
     }
   }else{
     sharedStrings <- ""
@@ -250,18 +278,16 @@ read.xlsx.default <- function(xlsxFile,
   }
   
   ## single function get all r, s (if detect dates is TRUE), t, v
-  cell_info <- .Call("openxlsx_getCellInfo",
-                     xmlFile = worksheet,
-                     sharedStrings = sharedStrings,
-                     skipEmptyRows = skipEmptyRows,
-                     startRow = startRow,
-                     rows = rows,
-                     getDates = detectDates,
-                     PACKAGE = "openxlsx")
-
+  cell_info <- getCellInfo(xmlFile = worksheet,
+                           sharedStrings = sharedStrings,
+                           skipEmptyRows = skipEmptyRows,
+                           startRow = startRow,
+                           rows = rows,
+                           getDates = detectDates)
+  
   if(fillMergedCells & length(cell_info$cellMerge) > 0){
     
-    stop("Not implemented")
+    # stop("Not implemented")
     
     merge_mapping <- mergeCell2mapping(cell_info$cellMerge)
     
@@ -286,37 +312,40 @@ read.xlsx.default <- function(xlsxFile,
     ## String refs (must sort)
     new_string_refs <- merge_mapping$ref[merge_mapping$anchor_cell %in% cell_info$string_refs]
     cell_info$string_refs <- c(cell_info$string_refs, new_string_refs)
-    cell_info$string_refs <- cell_info$string_refs[order(as.integer(gsub("[A-Z]", "", cell_info$string_refs)), nchar(cell_info$string_refs), cell_info$string_refs)]
+    cell_info$string_refs <- cell_info$string_refs[order(as.integer(gsub("[A-Z]", "", cell_info$string_refs, perl = TRUE)), nchar(cell_info$string_refs), cell_info$string_refs)]
     
     ## r
     cell_info$r <- c(cell_info$r, merge_mapping$ref)
     cell_info$v <- c(cell_info$v, cell_info$v[inds])
     
-    ord <- order(as.integer(gsub("[A-Z]", "", cell_info$r)), nchar(cell_info$r), cell_info$r)
+    ord <- order(as.integer(gsub(pattern = "[A-Z]", replacement = "", x = cell_info$r, perl = TRUE)), nchar(cell_info$r), cell_info$r)
     
     cell_info$r <- cell_info$r[ord]
     cell_info$v <- cell_info$v[ord]
+    if(length(cell_info$s) > 0)
+      cell_info$s <- c(cell_info$s, cell_info$s[inds])[ord]
     
-    if(length(cell_info$s) > 0){
-      cell_info$s <- c(cell_info$s, cell_info$s[inds])
-      cell_info$s <- cell_info$s[ord]
-    }
-    
-    cell_info$nRows <- .Call("openxlsx_calc_number_rows", cell_info$r, skipEmptyRows, PACKAGE = "openxlsx")
+  
+    cell_info$nRows <- calc_number_rows(x =  cell_info$r, skipEmptyRows = skipEmptyRows)
     
   }
   
   
-
+  
   cell_rows <- as.integer(gsub("[A-Z]", "", cell_info$r, perl = TRUE))
-  cell_cols <- .Call('openxlsx_convert_from_excel_ref', PACKAGE = 'openxlsx', cell_info$r)
-
-
+  cell_cols <- convert_from_excel_ref(x = cell_info$r)
+  
+  
   ######################################################################
   ## subsetting
   
-  keep <- !is.na(cell_info$v)
+  ## Remove cells where cell is NA (na.strings or empty sharedString '<si><t/></si>')
+  if(length(cell_info$v) == 0){
+    warning("No data found on worksheet.", call. = FALSE)
+    return(NULL)
+  }
   
+  keep <- !is.na(cell_info$v)
   if(!is.null(cols))
     keep <- keep & (cell_cols %in% cols)
   
@@ -336,6 +365,8 @@ read.xlsx.default <- function(xlsxFile,
   
   if(skipEmptyRows){
     nRows <- length(unique(cell_rows))
+  }else if(reading_named_region){ ## keep region the correct size
+    nRows <- max(rows) - min(rows) + 1;
   }else{
     nRows <- max(cell_rows) - min(cell_rows) + 1;
   }
@@ -361,7 +392,7 @@ read.xlsx.default <- function(xlsxFile,
         string_refs <- string_refs[!string_refs %in% r[toRemove]]
         v <- v[-toRemove]
         r <- r[-toRemove]
-        nRows <- .Call("openxlsx_calc_number_rows", r, skipEmptyRows, PACKAGE = "openxlsx")
+        nRows <- calc_number_rows(x =  r, skipEmptyRows = skipEmptyRows)
       }
     }
   }
@@ -380,28 +411,28 @@ read.xlsx.default <- function(xlsxFile,
     styles <- removeHeadTag(styles)
     
     ## Number formats
-    numFmts <- .Call("openxlsx_getChildlessNode", styles, "<numFmt ", PACKAGE = "openxlsx")
+    numFmts <- getChildlessNode(xml = styles, tag = "<numFmt ")
     
     dateIds <- NULL
     if(length(numFmts) > 0){
       
-      numFmtsIds <- sapply(numFmts, function(x) .Call("openxlsx_getAttr", x, 'numFmtId="', PACKAGE = "openxlsx"), USE.NAMES = FALSE)
-      formatCodes <- sapply(numFmts, function(x) .Call("openxlsx_getAttr", x, 'formatCode="', PACKAGE = "openxlsx"), USE.NAMES = FALSE)
+      numFmtsIds <- sapply(numFmts, getAttr, tag = 'numFmtId="', USE.NAMES = FALSE)
+      formatCodes <- sapply(numFmts, getAttr, tag = 'formatCode="', USE.NAMES = FALSE)
       formatCodes <- gsub(".*(?<=\\])|@", "", formatCodes, perl = TRUE)
       
       ## this regex defines what "looks" like a date
       dateIds <- numFmtsIds[!grepl("[^mdyhsapAMP[:punct:] ]", formatCodes) & nchar(formatCodes > 3)]
       
     }     
-
+    
     dateIds <- c(dateIds, 14)
     
     ## which styles are using these dateIds
-    cellXfs <- .Call("openxlsx_getNodes", styles, "<cellXfs", PACKAGE = "openxlsx") 
-    xf <- .Call("openxlsx_getChildlessNode", cellXfs, "<xf ", PACKAGE = "openxlsx")
+    cellXfs <- getNodes(xml = styles, tagIn = "<cellXfs") 
+    xf <- getChildlessNode(xml = cellXfs, tag = "<xf ")
     lookingFor <- paste(sprintf('numFmtId="%s"', dateIds), collapse = "|")
     dateStyleIds <- which(sapply(xf, function(x) grepl(lookingFor, x), USE.NAMES = FALSE)) - 1L
-
+    
     isDate <- (s %in% dateStyleIds)
     
     ## set to false if in string_refs
@@ -413,27 +444,26 @@ read.xlsx.default <- function(xlsxFile,
     not_an_integer <- (not_an_integer %% 1L != 0) & !is.na(not_an_integer)
     isDate[not_an_integer] <- FALSE
     
-
+    
     ## perform int to date to character convertsion (way too slow)
     v[isDate] <- format(as.Date(as.integer(v[isDate]) - origin, origin = "1970-01-01"), "%Y-%m-%d")
     
   }else{
     isDate <- as.logical(NA)
   }
-
+  
   ## Build data.frame
-  m <- .Call("openxlsx_read_workbook"
-             , cell_cols
-             , cell_rows
-             , v
-             , string_refs
-             , isDate
-             , colNames
-             , skipEmptyRows
-             , skipEmptyCols
-             , nRows
-             , clean_names
-             , PACKAGE = "openxlsx")
+  m <- read_workbook(cols_in = cell_cols,
+                     rows_in = cell_rows,
+                     v = v,
+                     string_inds = string_refs,
+                     is_date = isDate,
+                     hasColNames = colNames,
+                     hasSepNames = sep.names,
+                     skipEmptyRows = skipEmptyRows,
+                     skipEmptyCols = skipEmptyCols,
+                     nRows = nRows,
+                     clean_names = clean_names)
   
   
   if(colNames && check.names)
@@ -468,11 +498,13 @@ read.xlsx.default <- function(xlsxFile,
 #' @seealso \code{\link{read.xlsx}}
 #' @export
 #' @examples
-#' xlsxFile <- system.file("readTest.xlsx", package = "openxlsx")
+#' xlsxFile <- system.file("extdata","readTest.xlsx", package = "openxlsx")
 #' df1 <- readWorkbook(xlsxFile = xlsxFile, sheet = 1)
 #' 
-#' xlsxFile <- system.file("readTest.xlsx", package = "openxlsx")
+#' xlsxFile <- system.file("extdata","readTest.xlsx", package = "openxlsx")
 #' df1 <- readWorkbook(xlsxFile = xlsxFile, sheet = 1, rows = c(1, 3, 5), cols = 1:3)
+#' 
+#' 
 readWorkbook <- function(xlsxFile,
                          sheet = 1,
                          startRow = 1,
@@ -484,6 +516,7 @@ readWorkbook <- function(xlsxFile,
                          rows = NULL,
                          cols = NULL,
                          check.names = FALSE,
+                         sep.names = ".",
                          namedRegion = NULL,
                          na.strings = "NA",
                          fillMergedCells = FALSE){
@@ -499,6 +532,7 @@ readWorkbook <- function(xlsxFile,
             rows = rows,
             cols = cols,
             check.names = check.names,
+            sep.names = ".",
             namedRegion = namedRegion,
             na.strings = na.strings,
             fillMergedCells = fillMergedCells)
